@@ -96,16 +96,10 @@
 #include "gtkplugin.h"
 #include "gtkutils.h"
 
-#ifndef _WIN32
-#include <X11/Xatom.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#endif
-
 #define NOTIFY_PLUGIN_ID "gtk-x11-notify"
 
 static PurplePlugin *my_plugin = NULL;
-#ifndef _WIN32
+#ifdef HAVE_X11
 static GdkAtom _Cardinal = GDK_NONE;
 static GdkAtom _PurpleUnseenCount = GDK_NONE;
 #endif
@@ -303,7 +297,6 @@ static int
 attach_signals(PurpleConversation *conv)
 {
 	PidginConversation *gtkconv = NULL;
-	PidginWindow *gtkwin = NULL;
 	GSList *imhtml_ids = NULL, *entry_ids = NULL;
 	guint id;
 
@@ -313,8 +306,6 @@ attach_signals(PurpleConversation *conv)
 		return 0;
 	}
 
-	gtkwin  = gtkconv->win;
-
 	if (purple_prefs_get_bool("/plugins/gtk/X11/notify/notify_focus")) {
 		/* TODO should really find a way to make this work no matter
 		 * where the focus is inside the conv window, without having
@@ -358,13 +349,11 @@ static void
 detach_signals(PurpleConversation *conv)
 {
 	PidginConversation *gtkconv = NULL;
-	PidginWindow *gtkwin = NULL;
 	GSList *ids = NULL, *l;
 
 	gtkconv = PIDGIN_CONVERSATION(conv);
 	if (!gtkconv)
 		return;
-	gtkwin  = gtkconv->win;
 
 	ids = purple_conversation_get_data(conv, "notify-imhtml-signals");
 	for (l = ids; l != NULL; l = l->next)
@@ -530,7 +519,7 @@ handle_count_title(PidginWindow *purplewin)
 static void
 handle_count_xprop(PidginWindow *purplewin)
 {
-#ifndef _WIN32
+#ifdef HAVE_X11
 	guint count;
 	GtkWidget *window;
 	GdkWindow *gdkwin;
@@ -650,7 +639,6 @@ static void
 apply_method()
 {
 	GList *convs;
-	PidginWindow *purplewin = NULL;
 
 	for (convs = purple_get_conversations(); convs != NULL;
 	     convs = convs->next) {
@@ -659,7 +647,6 @@ apply_method()
 		/* remove notifications */
 		unnotify(conv, FALSE);
 
-		purplewin = PIDGIN_CONVERSATION(conv)->win;
 		if (GPOINTER_TO_INT(purple_conversation_get_data(conv, "notify-message-count")) != 0)
 			/* reattach appropriate notifications */
 			notify(conv, FALSE);
@@ -764,7 +751,7 @@ get_config_frame(PurplePlugin *plugin)
 	g_signal_connect(G_OBJECT(toggle), "toggled",
 	                 G_CALLBACK(method_toggle_cb), "method_count");
 
-#ifndef _WIN32
+#ifdef HAVE_X11
 	/* Count xprop method button */
 	toggle = gtk_check_button_new_with_mnemonic(_("Insert count of new message into _X property"));
 	gtk_box_pack_start(GTK_BOX(vbox), toggle, FALSE, FALSE, 0);
